@@ -12,20 +12,17 @@
 //! use openssl::bn::BigNum;
 //! use openssl::error::ErrorStack;
 //!
-//! fn bignums() -> Result<(), ErrorStack> {
+//! fn main() -> Result<(), ErrorStack> {
 //!   let a = BigNum::new()?; // a = 0
 //!   let b = BigNum::from_dec_str("1234567890123456789012345")?;
 //!   let c = &a * &b;
 //!   assert_eq!(a, c);
 //!   Ok(())
 //! }
-//! # fn main() {
-//! #    bignums();
-//! # }
 //! ```
 //!
 //! [`BIGNUM`]: https://wiki.openssl.org/index.php/Manual:Bn_internal(3)
-use ffi;
+use cfg_if::cfg_if;
 use foreign_types::{ForeignType, ForeignTypeRef};
 use libc::c_int;
 use std::cmp::Ordering;
@@ -33,10 +30,10 @@ use std::ffi::CString;
 use std::ops::{Add, Deref, Div, Mul, Neg, Rem, Shl, Shr, Sub};
 use std::{fmt, ptr};
 
-use asn1::Asn1Integer;
-use error::ErrorStack;
-use string::OpensslString;
-use {cvt, cvt_n, cvt_p};
+use crate::asn1::Asn1Integer;
+use crate::error::ErrorStack;
+use crate::string::OpensslString;
+use crate::{cvt, cvt_n, cvt_p};
 
 cfg_if! {
     if #[cfg(ossl110)] {
@@ -117,13 +114,13 @@ foreign_type_and_impl_send_sync! {
     type CType = ffi::BIGNUM;
     fn drop = ffi::BN_free;
 
-    /// Dynamically sized large number impelementation
+    /// Dynamically sized large number implementation
     ///
     /// Perform large number mathematics.  Create a new BigNum
     /// with [`new`].  Perform standard mathematics on large numbers using
     /// methods from [`Dref<Target = BigNumRef>`]
     ///
-    /// OpenSSL documenation at [`BN_new`].
+    /// OpenSSL documentation at [`BN_new`].
     ///
     /// [`new`]: struct.BigNum.html#method.new
     /// [`Dref<Target = BigNumRef>`]: struct.BigNum.html#deref-methods
@@ -191,6 +188,7 @@ impl BigNumRef {
     /// OpenSSL documentation at [`BN_div_word`]
     ///
     /// [`BN_div_word`]: https://www.openssl.org/docs/man1.1.0/crypto/BN_div_word.html
+    #[allow(clippy::useless_conversion)]
     pub fn div_word(&mut self, w: u32) -> Result<u64, ErrorStack> {
         unsafe {
             let r = ffi::BN_div_word(self.as_ptr(), w.into());
@@ -207,6 +205,7 @@ impl BigNumRef {
     /// OpenSSL documentation at [`BN_mod_word`]
     ///
     /// [`BN_mod_word`]: https://www.openssl.org/docs/man1.1.0/crypto/BN_mod_word.html
+    #[allow(clippy::useless_conversion)]
     pub fn mod_word(&self, w: u32) -> Result<u64, ErrorStack> {
         unsafe {
             let r = ffi::BN_mod_word(self.as_ptr(), w.into());
@@ -244,6 +243,7 @@ impl BigNumRef {
     /// OpenSSL documentation at [`BN_set_bit`]
     ///
     /// [`BN_set_bit`]: https://www.openssl.org/docs/man1.1.0/crypto/BN_set_bit.html
+    #[allow(clippy::useless_conversion)]
     pub fn set_bit(&mut self, n: i32) -> Result<(), ErrorStack> {
         unsafe { cvt(ffi::BN_set_bit(self.as_ptr(), n.into())).map(|_| ()) }
     }
@@ -255,6 +255,7 @@ impl BigNumRef {
     /// OpenSSL documentation at [`BN_clear_bit`]
     ///
     /// [`BN_clear_bit`]: https://www.openssl.org/docs/man1.1.0/crypto/BN_clear_bit.html
+    #[allow(clippy::useless_conversion)]
     pub fn clear_bit(&mut self, n: i32) -> Result<(), ErrorStack> {
         unsafe { cvt(ffi::BN_clear_bit(self.as_ptr(), n.into())).map(|_| ()) }
     }
@@ -264,6 +265,7 @@ impl BigNumRef {
     /// OpenSSL documentation at [`BN_is_bit_set`]
     ///
     /// [`BN_is_bit_set`]: https://www.openssl.org/docs/man1.1.0/crypto/BN_is_bit_set.html
+    #[allow(clippy::useless_conversion)]
     pub fn is_bit_set(&self, n: i32) -> bool {
         unsafe { ffi::BN_is_bit_set(self.as_ptr(), n.into()) == 1 }
     }
@@ -275,6 +277,7 @@ impl BigNumRef {
     /// OpenSSL documentation at [`BN_mask_bits`]
     ///
     /// [`BN_mask_bits`]: https://www.openssl.org/docs/man1.1.0/crypto/BN_mask_bits.html
+    #[allow(clippy::useless_conversion)]
     pub fn mask_bits(&mut self, n: i32) -> Result<(), ErrorStack> {
         unsafe { cvt(ffi::BN_mask_bits(self.as_ptr(), n.into())).map(|_| ()) }
     }
@@ -322,6 +325,7 @@ impl BigNumRef {
     /// OpenSSL documentation at [`BN_lshift`]
     ///
     /// [`BN_lshift`]: https://www.openssl.org/docs/man1.1.0/crypto/BN_lshift.html
+    #[allow(clippy::useless_conversion)]
     pub fn lshift(&mut self, a: &BigNumRef, n: i32) -> Result<(), ErrorStack> {
         unsafe { cvt(ffi::BN_lshift(self.as_ptr(), a.as_ptr(), n.into())).map(|_| ()) }
     }
@@ -331,6 +335,7 @@ impl BigNumRef {
     /// OpenSSL documentation at [`BN_rshift`]
     ///
     /// [`BN_rshift`]: https://www.openssl.org/docs/man1.1.0/crypto/BN_rshift.html
+    #[allow(clippy::useless_conversion)]
     pub fn rshift(&mut self, a: &BigNumRef, n: i32) -> Result<(), ErrorStack> {
         unsafe { cvt(ffi::BN_rshift(self.as_ptr(), a.as_ptr(), n.into())).map(|_| ()) }
     }
@@ -416,6 +421,7 @@ impl BigNumRef {
     ///
     /// [`constants`]: index.html#constants
     /// [`BN_rand`]: https://www.openssl.org/docs/man1.1.0/crypto/BN_rand.html
+    #[allow(clippy::useless_conversion)]
     pub fn rand(&mut self, bits: i32, msb: MsbOption, odd: bool) -> Result<(), ErrorStack> {
         unsafe {
             cvt(ffi::BN_rand(
@@ -433,6 +439,7 @@ impl BigNumRef {
     /// OpenSSL documentation at [`BN_psuedo_rand`]
     ///
     /// [`BN_psuedo_rand`]: https://www.openssl.org/docs/man1.1.0/crypto/BN_pseudo_rand.html
+    #[allow(clippy::useless_conversion)]
     pub fn pseudo_rand(&mut self, bits: i32, msb: MsbOption, odd: bool) -> Result<(), ErrorStack> {
         unsafe {
             cvt(ffi::BN_pseudo_rand(
@@ -811,7 +818,7 @@ impl BigNumRef {
     /// # Return Value
     ///
     /// Returns `true` if `self` is prime with an error probability of less than `0.25 ^ checks`.
-
+    #[allow(clippy::useless_conversion)]
     pub fn is_prime(&self, checks: i32, ctx: &mut BigNumContextRef) -> Result<bool, ErrorStack> {
         unsafe {
             cvt_n(ffi::BN_is_prime_ex(
@@ -837,6 +844,7 @@ impl BigNumRef {
     /// # Return Value
     ///
     /// Returns `true` if `self` is prime with an error probability of less than `0.25 ^ checks`.
+    #[allow(clippy::useless_conversion)]
     pub fn is_prime_fasttest(
         &self,
         checks: i32,
@@ -1114,7 +1122,7 @@ impl BigNum {
 }
 
 impl fmt::Debug for BigNumRef {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self.to_dec_str() {
             Ok(s) => f.write_str(&s),
             Err(e) => Err(e.into()),
@@ -1123,7 +1131,7 @@ impl fmt::Debug for BigNumRef {
 }
 
 impl fmt::Debug for BigNum {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self.to_dec_str() {
             Ok(s) => f.write_str(&s),
             Err(e) => Err(e.into()),
@@ -1132,7 +1140,7 @@ impl fmt::Debug for BigNum {
 }
 
 impl fmt::Display for BigNumRef {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self.to_dec_str() {
             Ok(s) => f.write_str(&s),
             Err(e) => Err(e.into()),
@@ -1141,7 +1149,7 @@ impl fmt::Display for BigNumRef {
 }
 
 impl fmt::Display for BigNum {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self.to_dec_str() {
             Ok(s) => f.write_str(&s),
             Err(e) => Err(e.into()),
@@ -1368,20 +1376,20 @@ impl Neg for BigNum {
 
 #[cfg(test)]
 mod tests {
-    use bn::{BigNum, BigNumContext};
+    use crate::bn::{BigNum, BigNumContext};
 
     #[test]
     fn test_to_from_slice() {
-        let v0 = BigNum::from_u32(10203004).unwrap();
+        let v0 = BigNum::from_u32(10_203_004).unwrap();
         let vec = v0.to_vec();
         let v1 = BigNum::from_slice(&vec).unwrap();
 
-        assert!(v0 == v1);
+        assert_eq!(v0, v1);
     }
 
     #[test]
     fn test_negation() {
-        let a = BigNum::from_u32(909829283).unwrap();
+        let a = BigNum::from_u32(909_829_283).unwrap();
 
         assert!(!a.is_negative());
         assert!((-a).is_negative());
@@ -1389,15 +1397,14 @@ mod tests {
 
     #[test]
     fn test_shift() {
-        let a = BigNum::from_u32(909829283).unwrap();
-        use std::ops::{Shl, Shr};
+        let a = BigNum::from_u32(909_829_283).unwrap();
 
-        assert!(a == a.shl(1).shr(1));
+        assert_eq!(a, &(&a << 1) >> 1);
     }
 
     #[test]
     fn test_rand_range() {
-        let range = BigNum::from_u32(909829283).unwrap();
+        let range = BigNum::from_u32(909_829_283).unwrap();
         let mut result = BigNum::from_dec_str(&range.to_dec_str().unwrap()).unwrap();
         range.rand_range(&mut result).unwrap();
         assert!(result >= BigNum::from_u32(0).unwrap() && result < range);
@@ -1405,7 +1412,7 @@ mod tests {
 
     #[test]
     fn test_pseudo_rand_range() {
-        let range = BigNum::from_u32(909829283).unwrap();
+        let range = BigNum::from_u32(909_829_283).unwrap();
         let mut result = BigNum::from_dec_str(&range.to_dec_str().unwrap()).unwrap();
         range.pseudo_rand_range(&mut result).unwrap();
         assert!(result >= BigNum::from_u32(0).unwrap() && result < range);
@@ -1413,7 +1420,7 @@ mod tests {
 
     #[test]
     fn test_prime_numbers() {
-        let a = BigNum::from_u32(19029017).unwrap();
+        let a = BigNum::from_u32(19_029_017).unwrap();
         let mut p = BigNum::new().unwrap();
         p.generate_prime(128, true, None, Some(&a)).unwrap();
 
