@@ -29,37 +29,35 @@
 //! Generate a 2048-bit RSA public/private key pair and print the public key.
 //!
 //! ```rust
-//!
-//! extern crate openssl;
-//!
 //! use openssl::rsa::Rsa;
 //! use openssl::pkey::PKey;
 //! use std::str;
 //!
-//! fn main() {
-//!     let rsa = Rsa::generate(2048).unwrap();
-//!     let pkey = PKey::from_rsa(rsa).unwrap();
+//! let rsa = Rsa::generate(2048).unwrap();
+//! let pkey = PKey::from_rsa(rsa).unwrap();
 //!
-//!     let pub_key: Vec<u8> = pkey.public_key_to_pem().unwrap();
-//!     println!("{:?}", str::from_utf8(pub_key.as_slice()).unwrap());
-//! }
+//! let pub_key: Vec<u8> = pkey.public_key_to_pem().unwrap();
+//! println!("{:?}", str::from_utf8(pub_key.as_slice()).unwrap());
 //! ```
 
-use ffi;
+use cfg_if::cfg_if;
 use foreign_types::{ForeignType, ForeignTypeRef};
 use libc::{c_int, c_long};
+use std::convert::TryFrom;
 use std::ffi::CString;
+use std::fmt;
 use std::mem;
 use std::ptr;
 
-use bio::MemBioSlice;
-use dh::Dh;
-use dsa::Dsa;
-use ec::EcKey;
-use error::ErrorStack;
-use rsa::Rsa;
-use util::{invoke_passwd_cb, CallbackState};
-use {cvt, cvt_p};
+use crate::bio::{MemBio, MemBioSlice};
+use crate::dh::Dh;
+use crate::dsa::Dsa;
+use crate::ec::EcKey;
+use crate::error::ErrorStack;
+use crate::rsa::Rsa;
+use crate::symm::Cipher;
+use crate::util::{invoke_passwd_cb, CallbackState};
+use crate::{cvt, cvt_p};
 
 /// A tag type indicating that a key only has parameters.
 pub enum Params {}
@@ -75,16 +73,6 @@ pub enum Private {}
 pub struct Id(c_int);
 
 impl Id {
-    /// Creates a `Id` from an integer representation.
-    pub fn from_raw(value: c_int) -> Id {
-        Id(value)
-    }
-
-    /// Returns the integer representation of the `Id`.
-    pub fn as_raw(&self) -> c_int {
-        self.0
-    }
-
     pub const RSA: Id = Id(ffi::EVP_PKEY_RSA);
     pub const HMAC: Id = Id(ffi::EVP_PKEY_HMAC);
     pub const DSA: Id = Id(ffi::EVP_PKEY_DSA);
@@ -95,6 +83,21 @@ impl Id {
     pub const ED25519: Id = Id(ffi::EVP_PKEY_ED25519);
     #[cfg(ossl111)]
     pub const ED448: Id = Id(ffi::EVP_PKEY_ED448);
+    #[cfg(ossl111)]
+    pub const X25519: Id = Id(ffi::EVP_PKEY_X25519);
+    #[cfg(ossl111)]
+    pub const X448: Id = Id(ffi::EVP_PKEY_X448);
+
+    /// Creates a `Id` from an integer representation.
+    pub fn from_raw(value: c_int) -> Id {
+        Id(value)
+    }
+
+    /// Returns the integer representation of the `Id`.
+    #[allow(clippy::trivially_copy_pass_by_ref)]
+    pub fn as_raw(&self) -> c_int {
+        self.0
+    }
 }
 
 /// A trait indicating that a key has parameters.
@@ -246,6 +249,34 @@ where
     {
         unsafe { ffi::EVP_PKEY_cmp(self.as_ptr(), other.as_ptr()) == 1 }
     }
+
+    /// Raw byte representation of a public key
+    ///
+    /// This function only works for algorithms that support raw public keys.
+    /// Currently this is: X25519, ED25519, X448 or ED448
+    ///
+    /// This corresponds to [`EVP_PKEY_get_raw_public_key`].
+    ///
+    /// [`EVP_PKEY_get_raw_public_key`]: https://www.openssl.org/docs/man1.1.1/man3/EVP_PKEY_get_raw_public_key.html
+    #[cfg(ossl111)]
+    pub fn raw_public_key(&self) -> Result<Vec<u8>, ErrorStack> {
+        unsafe {
+            let mut len = 0;
+            cvt(ffi::EVP_PKEY_get_raw_public_key(
+                self.as_ptr(),
+                ptr::null_mut(),
+                &mut len,
+            ))?;
+            let mut buf = vec![0u8; len];
+            cvt(ffi::EVP_PKEY_get_raw_public_key(
+                self.as_ptr(),
+                buf.as_mut_ptr(),
+                &mut len,
+            ))?;
+            buf.truncate(len);
+            Ok(buf)
+        }
+    }
 }
 
 impl<T> PKeyRef<T>
@@ -281,6 +312,82 @@ where
         private_key_to_der,
         ffi::i2d_PrivateKey
     }
+
+    /// Raw byte representation of a private key
+    ///
+    /// This function only works for algorithms that support raw private keys.
+    /// Currently this is: HMAC, X25519, ED25519, X448 or ED448
+    ///
+    /// This corresponds to [`EVP_PKEY_get_raw_private_key`].
+    ///
+    /// [`EVP_PKEY_get_raw_private_key`]: https://www.openssl.org/docs/man1.1.1/man3/EVP_PKEY_get_raw_private_key.html
+    #[cfg(ossl111)]
+    pub fn raw_private_key(&self) -> Result<Vec<u8>, ErrorStack> {
+        unsafe {
+            let mut len = 0;
+            cvt(ffi::EVP_PKEY_get_raw_private_key(
+                self.as_ptr(),
+                ptr::null_mut(),
+                &mut len,
+            ))?;
+            let mut buf = vec![0u8; len];
+            cvt(ffi::EVP_PKEY_get_raw_private_key(
+                self.as_ptr(),
+                buf.as_mut_ptr(),
+                &mut len,
+            ))?;
+            buf.truncate(len);
+            Ok(buf)
+        }
+    }
+
+    /// Serializes a private key into a DER-formatted PKCS#8, using the supplied password to
+    /// encrypt the key.
+    ///
+    /// # Panics
+    ///
+    /// Panics if `passphrase` contains an embedded null.
+    pub fn private_key_to_pkcs8_passphrase(
+        &self,
+        cipher: Cipher,
+        passphrase: &[u8],
+    ) -> Result<Vec<u8>, ErrorStack> {
+        unsafe {
+            let bio = MemBio::new()?;
+            let len = passphrase.len();
+            let passphrase = CString::new(passphrase).unwrap();
+            cvt(ffi::i2d_PKCS8PrivateKey_bio(
+                bio.as_ptr(),
+                self.as_ptr(),
+                cipher.as_ptr(),
+                passphrase.as_ptr() as *const _ as *mut _,
+                len as ::libc::c_int,
+                None,
+                ptr::null_mut(),
+            ))?;
+
+            Ok(bio.get_buf().to_owned())
+        }
+    }
+}
+
+impl<T> fmt::Debug for PKey<T> {
+    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let alg = match self.id() {
+            Id::RSA => "RSA",
+            Id::HMAC => "HMAC",
+            Id::DSA => "DSA",
+            Id::DH => "DH",
+            Id::EC => "EC",
+            #[cfg(ossl111)]
+            Id::ED25519 => "Ed25519",
+            #[cfg(ossl111)]
+            Id::ED448 => "Ed448",
+            _ => "unknown",
+        };
+        fmt.debug_struct("PKey").field("algorithm", &alg).finish()
+        // TODO: Print details for each specific type of key
+    }
 }
 
 impl<T> Clone for PKey<T> {
@@ -394,7 +501,8 @@ impl PKey<Private> {
     ///
     /// To compute CMAC values, use the `sign` module.
     #[cfg(ossl110)]
-    pub fn cmac(cipher: &::symm::Cipher, key: &[u8]) -> Result<PKey<Private>, ErrorStack> {
+    #[allow(clippy::trivially_copy_pass_by_ref)]
+    pub fn cmac(cipher: &Cipher, key: &[u8]) -> Result<PKey<Private>, ErrorStack> {
         unsafe {
             assert!(key.len() <= c_int::max_value() as usize);
             let kctx = cvt_p(ffi::EVP_PKEY_CTX_new_id(
@@ -448,7 +556,7 @@ impl PKey<Private> {
         }
     }
 
-    #[cfg(ossl110)]
+    #[cfg(ossl111)]
     fn generate_eddsa(nid: c_int) -> Result<PKey<Private>, ErrorStack> {
         unsafe {
             let kctx = cvt_p(ffi::EVP_PKEY_CTX_new_id(nid, ptr::null_mut()))?;
@@ -470,6 +578,18 @@ impl PKey<Private> {
         }
     }
 
+    /// Generates a new private Ed25519 key
+    #[cfg(ossl111)]
+    pub fn generate_x25519() -> Result<PKey<Private>, ErrorStack> {
+        PKey::generate_eddsa(ffi::EVP_PKEY_X25519)
+    }
+
+    /// Generates a new private Ed448 key
+    #[cfg(ossl111)]
+    pub fn generate_x448() -> Result<PKey<Private>, ErrorStack> {
+        PKey::generate_eddsa(ffi::EVP_PKEY_X448)
+    }
+
     /// Generates a new private Ed25519 key
     #[cfg(ossl111)]
     pub fn generate_ed25519() -> Result<PKey<Private>, ErrorStack> {
@@ -527,10 +647,7 @@ impl PKey<Private> {
     /// Deserializes a DER-formatted PKCS#8 unencrypted private key.
     ///
     /// This method is mainly for interoperability reasons. Encrypted keyfiles should be preferred.
-    pub fn private_key_from_pkcs8(
-        der: &[u8],
-    ) -> Result<PKey<Private>, ErrorStack>
-    {
+    pub fn private_key_from_pkcs8(der: &[u8]) -> Result<PKey<Private>, ErrorStack> {
         unsafe {
             ffi::init();
             let len = der.len().min(c_long::max_value() as usize) as c_long;
@@ -539,8 +656,7 @@ impl PKey<Private> {
                 &mut der.as_ptr(),
                 len,
             ))?;
-            let res = cvt_p(ffi::EVP_PKCS82PKEY(p8inf))
-                .map(|p| PKey::from_ptr(p));
+            let res = cvt_p(ffi::EVP_PKCS82PKEY(p8inf)).map(|p| PKey::from_ptr(p));
             ffi::PKCS8_PRIV_KEY_INFO_free(p8inf);
             res
         }
@@ -595,6 +711,30 @@ impl PKey<Private> {
             .map(|p| PKey::from_ptr(p))
         }
     }
+
+    /// Creates a private key from its raw byte representation
+    ///
+    /// Algorithm types that support raw private keys are HMAC, X25519, ED25519, X448 or ED448
+    ///
+    /// This corresponds to [`EVP_PKEY_new_raw_private_key`].
+    ///
+    /// [`EVP_PKEY_new_raw_private_key`]: https://www.openssl.org/docs/man1.1.1/man3/EVP_PKEY_new_raw_private_key.html
+    #[cfg(ossl111)]
+    pub fn private_key_from_raw_bytes(
+        bytes: &[u8],
+        key_type: Id,
+    ) -> Result<PKey<Private>, ErrorStack> {
+        unsafe {
+            ffi::init();
+            cvt_p(ffi::EVP_PKEY_new_raw_private_key(
+                key_type.as_raw(),
+                ptr::null_mut(),
+                bytes.as_ptr(),
+                bytes.len(),
+            ))
+            .map(|p| PKey::from_ptr(p))
+        }
+    }
 }
 
 impl PKey<Public> {
@@ -621,6 +761,30 @@ impl PKey<Public> {
         PKey<Public>,
         ffi::d2i_PUBKEY
     }
+
+    /// Creates a public key from its raw byte representation
+    ///
+    /// Algorithm types that support raw public keys are X25519, ED25519, X448 or ED448
+    ///
+    /// This corresponds to [`EVP_PKEY_new_raw_public_key`].
+    ///
+    /// [`EVP_PKEY_new_raw_public_key`]: https://www.openssl.org/docs/man1.1.1/man3/EVP_PKEY_new_raw_public_key.html
+    #[cfg(ossl111)]
+    pub fn public_key_from_raw_bytes(
+        bytes: &[u8],
+        key_type: Id,
+    ) -> Result<PKey<Public>, ErrorStack> {
+        unsafe {
+            ffi::init();
+            cvt_p(ffi::EVP_PKEY_new_raw_public_key(
+                key_type.as_raw(),
+                ptr::null_mut(),
+                bytes.as_ptr(),
+                bytes.len(),
+            ))
+            .map(|p| PKey::from_ptr(p))
+        }
+    }
 }
 
 cfg_if! {
@@ -640,17 +804,86 @@ cfg_if! {
     }
 }
 
+impl<T> TryFrom<EcKey<T>> for PKey<T> {
+    type Error = ErrorStack;
+
+    fn try_from(ec_key: EcKey<T>) -> Result<PKey<T>, ErrorStack> {
+        PKey::from_ec_key(ec_key)
+    }
+}
+
+impl<T> TryFrom<PKey<T>> for EcKey<T> {
+    type Error = ErrorStack;
+
+    fn try_from(pkey: PKey<T>) -> Result<EcKey<T>, ErrorStack> {
+        pkey.ec_key()
+    }
+}
+
+impl<T> TryFrom<Rsa<T>> for PKey<T> {
+    type Error = ErrorStack;
+
+    fn try_from(rsa: Rsa<T>) -> Result<PKey<T>, ErrorStack> {
+        PKey::from_rsa(rsa)
+    }
+}
+
+impl<T> TryFrom<PKey<T>> for Rsa<T> {
+    type Error = ErrorStack;
+
+    fn try_from(pkey: PKey<T>) -> Result<Rsa<T>, ErrorStack> {
+        pkey.rsa()
+    }
+}
+
+impl<T> TryFrom<Dsa<T>> for PKey<T> {
+    type Error = ErrorStack;
+
+    fn try_from(dsa: Dsa<T>) -> Result<PKey<T>, ErrorStack> {
+        PKey::from_dsa(dsa)
+    }
+}
+
+impl<T> TryFrom<PKey<T>> for Dsa<T> {
+    type Error = ErrorStack;
+
+    fn try_from(pkey: PKey<T>) -> Result<Dsa<T>, ErrorStack> {
+        pkey.dsa()
+    }
+}
+
+impl<T> TryFrom<Dh<T>> for PKey<T> {
+    type Error = ErrorStack;
+
+    fn try_from(dh: Dh<T>) -> Result<PKey<T>, ErrorStack> {
+        PKey::from_dh(dh)
+    }
+}
+
+impl<T> TryFrom<PKey<T>> for Dh<T> {
+    type Error = ErrorStack;
+
+    fn try_from(pkey: PKey<T>) -> Result<Dh<T>, ErrorStack> {
+        pkey.dh()
+    }
+}
+
 #[cfg(test)]
 mod tests {
-    use dh::Dh;
-    use dsa::Dsa;
-    use ec::EcKey;
-    use nid::Nid;
-    use rsa::Rsa;
-    use symm::Cipher;
+    use std::convert::TryInto;
+
+    use crate::dh::Dh;
+    use crate::dsa::Dsa;
+    use crate::ec::EcKey;
+    use crate::nid::Nid;
+    use crate::rsa::Rsa;
+    use crate::symm::Cipher;
 
     use super::*;
 
+    #[cfg(ossl111)]
+    use crate::rand::rand_bytes;
+
     #[test]
     fn test_to_password() {
         let rsa = Rsa::generate(2048).unwrap();
@@ -672,6 +905,17 @@ mod tests {
     fn test_encrypted_pkcs8_passphrase() {
         let key = include_bytes!("../test/pkcs8.der");
         PKey::private_key_from_pkcs8_passphrase(key, b"mypass").unwrap();
+
+        let rsa = Rsa::generate(2048).unwrap();
+        let pkey = PKey::from_rsa(rsa).unwrap();
+        let der = pkey
+            .private_key_to_pkcs8_passphrase(Cipher::aes_128_cbc(), b"mypass")
+            .unwrap();
+        let pkey2 = PKey::private_key_from_pkcs8_passphrase(&der, b"mypass").unwrap();
+        assert_eq!(
+            pkey.private_key_to_der().unwrap(),
+            pkey2.private_key_to_der().unwrap()
+        );
     }
 
     #[test]
@@ -761,4 +1005,127 @@ mod tests {
         assert_eq!(pkey.id(), Id::EC);
         assert!(pkey.rsa().is_err());
     }
+
+    #[test]
+    fn test_rsa_conversion() {
+        let rsa = Rsa::generate(2048).unwrap();
+        let pkey: PKey<Private> = rsa.clone().try_into().unwrap();
+        let rsa_: Rsa<Private> = pkey.try_into().unwrap();
+        // Eq is missing
+        assert_eq!(rsa.p(), rsa_.p());
+        assert_eq!(rsa.q(), rsa_.q());
+    }
+
+    #[test]
+    fn test_dsa_conversion() {
+        let dsa = Dsa::generate(2048).unwrap();
+        let pkey: PKey<Private> = dsa.clone().try_into().unwrap();
+        let dsa_: Dsa<Private> = pkey.try_into().unwrap();
+        // Eq is missing
+        assert_eq!(dsa.priv_key(), dsa_.priv_key());
+    }
+
+    #[test]
+    fn test_ec_key_conversion() {
+        let group = crate::ec::EcGroup::from_curve_name(crate::nid::Nid::X9_62_PRIME256V1).unwrap();
+        let ec_key = EcKey::generate(&group).unwrap();
+        let pkey: PKey<Private> = ec_key.clone().try_into().unwrap();
+        let ec_key_: EcKey<Private> = pkey.try_into().unwrap();
+        // Eq is missing
+        assert_eq!(ec_key.private_key(), ec_key_.private_key());
+    }
+
+    #[test]
+    fn test_dh_conversion() {
+        let dh_params = include_bytes!("../test/dhparams.pem");
+        let dh_params = Dh::params_from_pem(dh_params).unwrap();
+        let dh = dh_params.generate_key().unwrap();
+
+        // Clone is missing for Dh, save the parameters
+        let p = dh.prime_p().to_owned().unwrap();
+        let q = dh.prime_q().map(|q| q.to_owned().unwrap());
+        let g = dh.generator().to_owned().unwrap();
+
+        let pkey: PKey<Private> = dh.try_into().unwrap();
+        let dh_: Dh<Private> = pkey.try_into().unwrap();
+
+        // Eq is missing
+        assert_eq!(&p, dh_.prime_p());
+        assert_eq!(q, dh_.prime_q().map(|q| q.to_owned().unwrap()));
+        assert_eq!(&g, dh_.generator());
+    }
+
+    #[cfg(ossl111)]
+    fn test_raw_public_key(gen: fn() -> Result<PKey<Private>, ErrorStack>, key_type: Id) {
+        // Generate a new key
+        let key = gen().unwrap();
+
+        // Get the raw bytes, and create a new key from the raw bytes
+        let raw = key.raw_public_key().unwrap();
+        let from_raw = PKey::public_key_from_raw_bytes(&raw, key_type).unwrap();
+
+        // Compare the der encoding of the original and raw / restored public key
+        assert_eq!(
+            key.public_key_to_der().unwrap(),
+            from_raw.public_key_to_der().unwrap()
+        );
+    }
+
+    #[cfg(ossl111)]
+    fn test_raw_private_key(gen: fn() -> Result<PKey<Private>, ErrorStack>, key_type: Id) {
+        // Generate a new key
+        let key = gen().unwrap();
+
+        // Get the raw bytes, and create a new key from the raw bytes
+        let raw = key.raw_private_key().unwrap();
+        let from_raw = PKey::private_key_from_raw_bytes(&raw, key_type).unwrap();
+
+        // Compare the der encoding of the original and raw / restored public key
+        assert_eq!(
+            key.private_key_to_der().unwrap(),
+            from_raw.private_key_to_der().unwrap()
+        );
+    }
+
+    #[cfg(ossl111)]
+    #[test]
+    fn test_raw_public_key_bytes() {
+        test_raw_public_key(PKey::generate_x25519, Id::X25519);
+        test_raw_public_key(PKey::generate_ed25519, Id::ED25519);
+        test_raw_public_key(PKey::generate_x448, Id::X448);
+        test_raw_public_key(PKey::generate_ed448, Id::ED448);
+    }
+
+    #[cfg(ossl111)]
+    #[test]
+    fn test_raw_private_key_bytes() {
+        test_raw_private_key(PKey::generate_x25519, Id::X25519);
+        test_raw_private_key(PKey::generate_ed25519, Id::ED25519);
+        test_raw_private_key(PKey::generate_x448, Id::X448);
+        test_raw_private_key(PKey::generate_ed448, Id::ED448);
+    }
+
+    #[cfg(ossl111)]
+    #[test]
+    fn test_raw_hmac() {
+        let mut test_bytes = vec![0u8; 32];
+        rand_bytes(&mut test_bytes).unwrap();
+
+        let hmac_key = PKey::hmac(&test_bytes).unwrap();
+        assert!(hmac_key.raw_public_key().is_err());
+
+        let key_bytes = hmac_key.raw_private_key().unwrap();
+        assert_eq!(key_bytes, test_bytes);
+    }
+
+    #[cfg(ossl111)]
+    #[test]
+    fn test_raw_key_fail() {
+        // Getting a raw byte representation will not work with Nist curves
+        let group = crate::ec::EcGroup::from_curve_name(Nid::SECP256K1).unwrap();
+        let ec_key = EcKey::generate(&group).unwrap();
+        let pkey = PKey::from_ec_key(ec_key).unwrap();
+        assert!(pkey.raw_private_key().is_err());
+        assert!(pkey.raw_public_key().is_err());
+    }
 }
